@@ -1,16 +1,46 @@
-{-# LANGUAGE TemplateHaskell, QuasiQuotes, OverloadedStrings, RecordWildCards  #-}
+{-# LANGUAGE TemplateHaskell, QuasiQuotes, OverloadedStrings, RecordWildCards #-}
 {-
 
 hledger-web's request handlers, and helpers.
 
 -}
 
-module Hledger.Web.Handlers where
+module Hledger.Web.Handlers
+(
+  -- * GET handlers
+  getRootR,
+  getJournalR,
+  getJournalEntriesR,
+  getJournalEditR,
+  getRegisterR,
+  -- ** helpers
+  -- sidebar,
+  -- accountsReportAsHtml,
+  -- accountQuery,
+  -- accountOnlyQuery,
+  -- accountUrl,
+  -- entriesReportAsHtml,
+  -- journalTransactionsReportAsHtml,
+  -- registerReportHtml,
+  -- registerItemsHtml,
+  -- registerChartHtml,
+  -- stringIfLongerThan,
+  -- numberTransactionsReportItems,
+  -- mixedAmountAsHtml,
+  -- * POST handlers
+  postJournalR,
+  postJournalEntriesR,
+  postJournalEditR,
+  postRegisterR,
+  -- * Common page components
+  -- * Utilities
+  ViewData(..),
+  nullviewdata,
+)
+where
 
 import Prelude
 import Control.Applicative ((<$>))
--- import Data.Aeson
-import Data.ByteString (ByteString)
 import Data.Either (lefts,rights)
 import Data.List
 import Data.Maybe
@@ -19,36 +49,39 @@ import qualified Data.Text (null)
 import Data.Time.Calendar
 import Data.Time.Clock
 import Data.Time.Format
-import System.FilePath (takeFileName, (</>))
+import System.FilePath (takeFileName)
 import System.IO.Storage (putValue, getValue)
 import System.Locale (defaultTimeLocale)
 import Text.Blaze (preEscapedString, toHtml)
-import Text.Hamlet hiding (hamletFile)
+import Text.Hamlet hiding (hamlet)
 import Text.Printf
 import Yesod.Core
 -- import Yesod.Json
 
-import Hledger hiding (today)
+import Hledger hiding (is)
 import Hledger.Cli hiding (version)
 import Hledger.Web.Foundation
 import Hledger.Web.Options
 import Hledger.Web.Settings
 
+-- routes:
+-- /static          StaticR         Static getStatic
+-- -- /favicon.ico     FaviconR        GET
+-- /robots.txt      RobotsR         GET
+-- /                RootR           GET
+-- /journal         JournalR        GET POST
+-- /journal/entries JournalEntriesR GET POST
+-- /journal/edit    JournalEditR    GET POST
+-- /register        RegisterR       GET POST
+-- -- /accounts        AccountsR       GET
+-- -- /api/accounts    AccountsJsonR   GET
 
--- getFaviconR :: Handler ()
--- getFaviconR = sendFile "image/x-icon" $ Hledger.Web.Settings.staticDir </> "favicon.ico"
-
--- getRobotsR :: Handler RepPlain
--- getRobotsR = return $ RepPlain $ toContent ("User-agent: *" :: ByteString)
+----------------------------------------------------------------------
+-- GET handlers
 
 getRootR :: Handler RepHtml
 getRootR = redirect defaultroute where defaultroute = RegisterR
 
-type AppRoute = Route App
-
-----------------------------------------------------------------------
--- main views:
-
 -- | The formatted journal view, with sidebar.
 getJournalR :: Handler RepHtml
 getJournalR = do
@@ -57,24 +90,49 @@ getJournalR = do
       -- XXX like registerReportAsHtml
       inacct = inAccount qopts
       -- injournal = isNothing inacct
-      filtering = m /= MatchAny
+      filtering = m /= Any
       -- showlastcolumn = if injournal && not filtering then False else True
       title = case inacct of
-                Nothing       -> "Journal"++filter
-                Just (a,subs) -> "Transactions in "++a++andsubs++filter
-                                  where andsubs = if subs then " (and subaccounts)" else ""
+                Nothing       -> "Journal"++s2
+                Just (a,inclsubs) -> "Transactions in "++a++s1++s2
+                                      where s1 = if inclsubs then " (and subaccounts)" else ""
                 where
-                  filter = if filtering then ", filtered" else ""
+                  s2 = if filtering then ", filtered" else ""
       maincontent = journalTransactionsReportAsHtml opts vd $ journalTransactionsReport (reportopts_ $ cliopts_ opts) j m
   defaultLayout $ do
       setTitle "hledger-web journal"
-      addHamlet [$hamlet|
+      addWidget $ toWidget [hamlet|
 ^{topbar vd}
-<div#content
- <div#sidebar
+<div#content>
+ <div#sidebar>
   ^{sidecontent}
- <div#main.register
-  <div#maincontent
+ <div#main.register>
+  <div#maincontent>
+   <h2#contenttitle>#{title}
+   ^{searchform vd}
+   ^{maincontent}
+  ^{addform vd}
+  ^{editform vd}
+  ^{importform}
+|]
+
+-- | The journal entries view, with sidebar.
+getJournalEntriesR :: Handler RepHtml
+getJournalEntriesR = do
+  vd@VD{..} <- getViewData
+  let
+      sidecontent = sidebar vd
+      title = "Journal entries" ++ if m /= Any then ", filtered" else "" :: String
+      maincontent = entriesReportAsHtml opts vd $ entriesReport (reportopts_ $ cliopts_ opts) Any $ filterJournalTransactions m j
+  defaultLayout $ do
+      setTitle "hledger-web journal"
+      addWidget $ toWidget [hamlet|
+^{topbar vd}
+<div#content>
+ <div#sidebar>
+  ^{sidecontent}
+ <div#main.journal>
+  <div#maincontent>
    <h2#contenttitle>#{title}
    ^{searchform vd}
    ^{maincontent}
@@ -89,42 +147,15 @@ getJournalEditR = do
   vd <- getViewData
   defaultLayout $ do
       setTitle "hledger-web journal edit form"
-      addHamlet $ editform vd
+      addWidget $ toWidget $ editform vd
 
--- | The journal entries view, with sidebar.
-getJournalEntriesR :: Handler RepHtml
-getJournalEntriesR = do
-  vd@VD{..} <- getViewData
-  let
-      sidecontent = sidebar vd
-      title = "Journal entries" ++ if m /= MatchAny then ", filtered" else "" :: String
-      maincontent = entriesReportAsHtml opts vd $ entriesReport (reportopts_ $ cliopts_ opts) nullfilterspec $ filterJournalTransactions2 m j
-  defaultLayout $ do
-      setTitle "hledger-web journal"
-      addHamlet [$hamlet|
-^{topbar vd}
-<div#content
- <div#sidebar
-  ^{sidecontent}
- <div#main.journal
-  <div#maincontent
-   <h2#contenttitle>#{title}
-   ^{searchform vd}
-   ^{maincontent}
-  ^{addform vd}
-  ^{editform vd}
-  ^{importform}
-|]
-
--- | The journal entries view, no sidebar.
-getJournalOnlyR :: Handler RepHtml
-getJournalOnlyR = do
-  vd@VD{..} <- getViewData
-  defaultLayout $ do
-      setTitle "hledger-web journal only"
-      addHamlet $ entriesReportAsHtml opts vd $ entriesReport (reportopts_ $ cliopts_ opts) nullfilterspec $ filterJournalTransactions2 m j
-
-----------------------------------------------------------------------
+-- -- | The journal entries view, no sidebar.
+-- getJournalOnlyR :: Handler RepHtml
+-- getJournalOnlyR = do
+--   vd@VD{..} <- getViewData
+--   defaultLayout $ do
+--       setTitle "hledger-web journal only"
+--       addWidget $ toWidget $ entriesReportAsHtml opts vd $ entriesReport (reportopts_ $ cliopts_ opts) nullfilterspec $ filterJournalTransactions2 m j
 
 -- | The main journal/account register view, with accounts sidebar.
 getRegisterR :: Handler RepHtml
@@ -132,22 +163,22 @@ getRegisterR = do
   vd@VD{..} <- getViewData
   let sidecontent = sidebar vd
       -- injournal = isNothing inacct
-      filtering = m /= MatchAny
-      title = "Transactions in "++a++andsubs++filter
+      filtering = m /= Any
+      title = "Transactions in "++a++s1++s2
                where
-                 (a,subs) = fromMaybe ("all accounts",False) $ inAccount qopts
-                 andsubs = if subs then " (and subaccounts)" else ""
-                 filter = if filtering then ", filtered" else ""
-      maincontent = registerReportHtml opts vd $ accountTransactionsReport (reportopts_ $ cliopts_ opts) j m $ fromMaybe MatchAny $ inAccountMatcher qopts
+                 (a,inclsubs) = fromMaybe ("all accounts",False) $ inAccount qopts
+                 s1 = if inclsubs then " (and subaccounts)" else ""
+                 s2 = if filtering then ", filtered" else ""
+      maincontent = registerReportHtml opts vd $ accountTransactionsReport (reportopts_ $ cliopts_ opts) j m $ fromMaybe Any $ inAccountQuery qopts
   defaultLayout $ do
       setTitle "hledger-web register"
-      addHamlet [$hamlet|
+      addWidget $ toWidget [hamlet|
 ^{topbar vd}
-<div#content
- <div#sidebar
+<div#content>
+ <div#sidebar>
   ^{sidecontent}
- <div#main.register
-  <div#maincontent
+ <div#main.register>
+  <div#maincontent>
    <h2#contenttitle>#{title}
    ^{searchform vd}
    ^{maincontent}
@@ -156,17 +187,15 @@ getRegisterR = do
   ^{importform}
 |]
 
--- | The register view, no sidebar.
-getRegisterOnlyR :: Handler RepHtml
-getRegisterOnlyR = do
-  vd@VD{..} <- getViewData
-  defaultLayout $ do
-      setTitle "hledger-web register only"
-      addHamlet $
-          case inAccountMatcher qopts of Just m' -> registerReportHtml opts vd $ accountTransactionsReport (reportopts_ $ cliopts_ opts) j m m'
-                                         Nothing -> registerReportHtml opts vd $ journalTransactionsReport (reportopts_ $ cliopts_ opts) j m
-
-----------------------------------------------------------------------
+-- -- | The register view, no sidebar.
+-- getRegisterOnlyR :: Handler RepHtml
+-- getRegisterOnlyR = do
+--   vd@VD{..} <- getViewData
+--   defaultLayout $ do
+--       setTitle "hledger-web register only"
+--       addWidget $ toWidget $
+--           case inAccountQuery qopts of Just m' -> registerReportHtml opts vd $ accountTransactionsReport (reportopts_ $ cliopts_ opts) j m m'
+--                                          Nothing -> registerReportHtml opts vd $ journalTransactionsReport (reportopts_ $ cliopts_ opts) j m
 
 {-
 -- | A simple accounts view. This one is json-capable, returning the chart
@@ -177,7 +206,7 @@ getAccountsR = do
   let j' = filterJournalPostings2 m j
       html = do
         setTitle "hledger-web accounts"
-        addHamlet $ accountsReportAsHtml opts vd $ accountsReport2 (reportopts_ $ cliopts_ opts) am j'
+        addWidget $ toWidget $ accountsReportAsHtml opts vd $ accountsReport2 (reportopts_ $ cliopts_ opts) am j'
       json = jsonMap [("accounts", toJSON $ journalAccountNames j')]
   defaultLayoutJson html json
 
@@ -189,39 +218,38 @@ getAccountsJsonR = do
   jsonToRepJson $ jsonMap [("accounts", toJSON $ journalAccountNames j')]
 -}
 
-----------------------------------------------------------------------
--- view helpers
+-- helpers
 
 -- | Render the sidebar used on most views.
 sidebar :: ViewData -> HtmlUrl AppRoute
-sidebar vd@VD{..} = accountsReportAsHtml opts vd $ accountsReport2 (reportopts_ $ cliopts_ opts) am j
+sidebar vd@VD{..} = accountsReportAsHtml opts vd $ accountsReport (reportopts_ $ cliopts_ opts) am j
 
--- | Render a "AccountsReport" as HTML.
+-- | Render an "AccountsReport" as html.
 accountsReportAsHtml :: WebOpts -> ViewData -> AccountsReport -> HtmlUrl AppRoute
 accountsReportAsHtml _ vd@VD{..} (items',total) =
- [$hamlet|
-<div#accountsheading
+ [hamlet|
+<div#accountsheading>
  <a#accounts-toggle-link.togglelink href="#" title="Toggle sidebar">[+]
-<div#accounts
+<div#accounts>
  <table.balancereport>
-  <tr
-   <td.add colspan=3
+  <tr>
+   <td.add colspan=3>
     <br>
     <a#addformlink href="#" onclick="return addformToggle(event)" title="Add a new transaction to the journal">Add a transaction..
 
-  <tr.item :allaccts:.inacct
-   <td.journal colspan=3
+  <tr.item :allaccts:.inacct>
+   <td.journal colspan=3>
     <br>
     <a href=@{JournalR} title="Show all transactions in journal format">Journal
-    <span.hoverlinks
+    <span.hoverlinks>
      &nbsp;
      <a href=@{JournalEntriesR} title="Show journal entries">entries
      &nbsp;
      <a#editformlink href="#" onclick="return editformToggle(event)" title="Edit the journal">
       edit
 
-  <tr
-   <td colspan=3
+  <tr>
+   <td colspan=3>
     <br>
     Accounts
 
@@ -236,17 +264,17 @@ accountsReportAsHtml _ vd@VD{..} (items',total) =
    <td>
 |]
  where
-   l = journalToLedger nullfilterspec j
-   inacctmatcher = inAccountMatcher qopts
+   l = journalToLedger Any j
+   inacctmatcher = inAccountQuery qopts
    allaccts = isNothing inacctmatcher
    items = items' -- maybe items' (\m -> filter (matchesAccount m . \(a,_,_,_)->a) items') showacctmatcher
    itemAsHtml :: ViewData -> AccountsReportItem -> HtmlUrl AppRoute
-   itemAsHtml _ (acct, adisplay, aindent, abal) = [$hamlet|
-<tr.item.#{inacctclass}
- <td.account.#{depthclass}
+   itemAsHtml _ (acct, adisplay, aindent, abal) = [hamlet|
+<tr.item.#{inacctclass}>
+ <td.account.#{depthclass}>
   #{indent}
   <a href="@?{acctquery}" title="Show transactions in this account, including subaccounts">#{adisplay}
-  <span.hoverlinks
+  <span.hoverlinks>
    $if hassubs
     &nbsp;
     <a href="@?{acctonlyquery}" title="Show transactions in this account only">only
@@ -262,7 +290,7 @@ accountsReportAsHtml _ vd@VD{..} (items',total) =
        numpostings = length $ apostings $ ledgerAccount l acct
        depthclass = "depth"++show aindent
        inacctclass = case inacctmatcher of
-                       Just m -> if m `matchesAccount` acct then "inacct" else "notinacct"
+                       Just m' -> if m' `matchesAccount` acct then "inacct" else "notinacct"
                        Nothing -> "" :: String
        indent = preEscapedString $ concat $ replicate (2 * (1+aindent)) "&nbsp;"
        acctquery = (RegisterR, [("q", pack $ accountQuery acct)])
@@ -274,19 +302,19 @@ accountQuery a = "inacct:" ++ quoteIfSpaced a -- (accountNameToAccountRegex a)
 accountOnlyQuery :: AccountName -> String
 accountOnlyQuery a = "inacctonly:" ++ quoteIfSpaced a -- (accountNameToAccountRegex a)
 
--- accountUrl :: AppRoute -> AccountName -> (AppRoute,[(String,ByteString)])
-accountUrl r a = (r, [("q",pack $ accountQuery a)])
+accountUrl :: AppRoute -> AccountName -> (AppRoute, [(Text, Text)])
+accountUrl r a = (r, [("q", pack $ accountQuery a)])
 
--- | Render a "EntriesReport" as HTML for the journal entries view.
+-- | Render an "EntriesReport" as html for the journal entries view.
 entriesReportAsHtml :: WebOpts -> ViewData -> EntriesReport -> HtmlUrl AppRoute
-entriesReportAsHtml _ vd items = [$hamlet|
+entriesReportAsHtml _ vd items = [hamlet|
 <table.journalreport>
  $forall i <- numbered items
   ^{itemAsHtml vd i}
  |]
  where
    itemAsHtml :: ViewData -> (Int, EntriesReportItem) -> HtmlUrl AppRoute
-   itemAsHtml _ (n, t) = [$hamlet|
+   itemAsHtml _ (n, t) = [hamlet|
 <tr.item.#{evenodd}>
  <td.transaction>
   <pre>#{txn}
@@ -295,11 +323,11 @@ entriesReportAsHtml _ vd items = [$hamlet|
        evenodd = if even n then "even" else "odd" :: String
        txn = trimnl $ showTransaction t where trimnl = reverse . dropWhile (=='\n') . reverse
 
--- | Render an "TransactionsReport" as HTML for the formatted journal view.
+-- | Render a "TransactionsReport" as html for the formatted journal view.
 journalTransactionsReportAsHtml :: WebOpts -> ViewData -> TransactionsReport -> HtmlUrl AppRoute
-journalTransactionsReportAsHtml _ vd (_,items) = [$hamlet|
-<table.journalreport
- <tr.headings
+journalTransactionsReportAsHtml _ vd (_,items) = [hamlet|
+<table.journalreport>
+ <tr.headings>
   <th.date align=left>Date
   <th.description align=left>Description
   <th.account align=left>Accounts
@@ -310,19 +338,19 @@ journalTransactionsReportAsHtml _ vd (_,items) = [$hamlet|
  where
 -- .#{datetransition}
    itemAsHtml :: ViewData -> (Int, Bool, Bool, Bool, TransactionsReportItem) -> HtmlUrl AppRoute
-   itemAsHtml VD{..} (n, _, _, _, (t, _, split, _, amt, _)) = [$hamlet|
-<tr.item.#{evenodd}.#{firstposting}
+   itemAsHtml VD{..} (n, _, _, _, (t, _, split, _, amt, _)) = [hamlet|
+<tr.item.#{evenodd}.#{firstposting}>
  <td.date>#{date}
  <td.description colspan=2 title="#{show t}">#{elideRight 60 desc}
  <td.amount align=right>
   $if showamt
    #{mixedAmountAsHtml amt}
-$forall p <- tpostings t
-  <tr.item.#{evenodd}.posting
-   <td.date
-   <td.description
-   <td.account>&nbsp;<a href="@?{accountUrl here $ paccount p}" title="Show transactions in #{paccount p}">#{elideRight 40 $ paccount p}
-   <td.amount align=right>#{mixedAmountAsHtml $ pamount p}
+$forall p' <- tpostings t
+  <tr.item.#{evenodd}.posting>
+   <td.date>
+   <td.description>
+   <td.account>&nbsp;<a href="@?{accountUrl here $ paccount p'}" title="Show transactions in #{paccount p'}">#{elideRight 40 $ paccount p'}
+   <td.amount align=right>#{mixedAmountAsHtml $ pamount p'}
 |]
      where
        evenodd = if even n then "even" else "odd" :: String
@@ -335,16 +363,16 @@ $forall p <- tpostings t
 
 -- Generate html for an account register, including a balance chart and transaction list.
 registerReportHtml :: WebOpts -> ViewData -> TransactionsReport -> HtmlUrl AppRoute
-registerReportHtml opts vd r@(_,items) = [$hamlet|
+registerReportHtml opts vd r@(_,items) = [hamlet|
  ^{registerChartHtml items}
  ^{registerItemsHtml opts vd r}
 |]
 
 -- Generate html for a transaction list from an "TransactionsReport".
 registerItemsHtml :: WebOpts -> ViewData -> TransactionsReport -> HtmlUrl AppRoute
-registerItemsHtml _ vd (balancelabel,items) = [$hamlet|
-<table.registerreport
- <tr.headings
+registerItemsHtml _ vd (balancelabel,items) = [hamlet|
+<table.registerreport>
+ <tr.headings>
   <th.date align=left>Date
   <th.description align=left>Description
   <th.account align=left>To/From Account
@@ -358,28 +386,28 @@ registerItemsHtml _ vd (balancelabel,items) = [$hamlet|
  |]
  where
    -- inacct = inAccount qopts
-   -- filtering = m /= MatchAny
+   -- filtering = m /= Any
    itemAsHtml :: ViewData -> (Int, Bool, Bool, Bool, TransactionsReportItem) -> HtmlUrl AppRoute
-   itemAsHtml VD{..} (n, newd, newm, _, (t, _, split, acct, amt, bal)) = [$hamlet|
-<tr.item.#{evenodd}.#{firstposting}.#{datetransition}
+   itemAsHtml VD{..} (n, newd, newm, _, (t, _, split, acct, amt, bal)) = [hamlet|
+<tr.item.#{evenodd}.#{firstposting}.#{datetransition}>
  <td.date>#{date}
  <td.description title="#{show t}">#{elideRight 30 desc}
- <td.account title="#{show t}"
-  <a
+ <td.account title="#{show t}">
+  <a>
    #{elideRight 40 acct}
   &nbsp;
-  <a.postings-toggle-link.togglelink href="#" title="Toggle all postings"
+  <a.postings-toggle-link.togglelink href="#" title="Toggle all postings">
    [+]
  <td.amount align=right>
   $if showamt
    #{mixedAmountAsHtml amt}
  <td.balance align=right>#{mixedAmountAsHtml bal}
-$forall p <- tpostings t
- <tr.item.#{evenodd}.posting style=#{postingsdisplaystyle}
-   <td.date
-   <td.description
-   <td.account>&nbsp;<a href="@?{accountUrl here $ paccount p}" title="Show transactions in #{paccount p}">#{elideRight 40 $ paccount p}
-   <td.amount align=right>#{mixedAmountAsHtml $ pamount p}
+$forall p' <- tpostings t
+ <tr.item.#{evenodd}.posting style=#{postingsdisplaystyle}>
+   <td.date>
+   <td.description>
+   <td.account>&nbsp;<a href="@?{accountUrl here $ paccount p'}" title="Show transactions in #{paccount p'}">#{elideRight 40 $ paccount p'}
+   <td.amount align=right>#{mixedAmountAsHtml $ pamount p'}
    <td.balance align=right>
 |]
      where
@@ -394,15 +422,22 @@ $forall p <- tpostings t
 
 -- | Generate javascript/html for a register balance line chart based on
 -- the provided "TransactionsReportItem"s.
+               -- registerChartHtml :: forall t (t1 :: * -> *) t2 t3 t4 t5.
+               --                      Data.Foldable.Foldable t1 =>
+               --                      t1 (Transaction, t2, t3, t4, t5, MixedAmount)
+               --                      -> t -> Text.Blaze.Internal.HtmlM ()
+registerChartHtml :: [TransactionsReportItem] -> HtmlUrl AppRoute
 registerChartHtml items =
  -- have to make sure plot is not called when our container (maincontent)
  -- is hidden, eg with add form toggled
- [$hamlet|
+ [hamlet|
+<div#register-chart style="width:600px;height:100px; margin-bottom:1em;">
 <script type=text/javascript>
- if (document.getElementById('maincontent').style.display != 'none')
-  \$(document).ready(function() {
-    /* render chart */
-      \$.plot($('#register-chart'),
+ \$(document).ready(function() {
+   /* render chart with flot, if visible */
+   var chartdiv = $('#register-chart');
+   if (chartdiv.is(':visible'))
+     \$.plot(chartdiv,
              [
               [
                $forall i <- items
@@ -417,15 +452,14 @@ registerChartHtml items =
              }
              );
   });
-<div#register-chart style="width:600px;height:100px; margin-bottom:1em;"
 |]
 
-stringIfLongerThan :: Int -> String -> String
-stringIfLongerThan n s = if length s > n then s else ""
+-- stringIfLongerThan :: Int -> String -> String
+-- stringIfLongerThan n s = if length s > n then s else ""
 
 numberTransactionsReportItems :: [TransactionsReportItem] -> [(Int,Bool,Bool,Bool,TransactionsReportItem)]
 numberTransactionsReportItems [] = []
-numberTransactionsReportItems is = number 0 nulldate is
+numberTransactionsReportItems items = number 0 nulldate items
   where
     number :: Int -> Day -> [TransactionsReportItem] -> [(Int,Bool,Bool,Bool,TransactionsReportItem)]
     number _ _ [] = []
@@ -437,13 +471,14 @@ numberTransactionsReportItems is = number 0 nulldate is
           (dy,dm,_) = toGregorian d
           (prevdy,prevdm,_) = toGregorian prevd
 
-mixedAmountAsHtml b = preEscapedString $ addclass $ intercalate "<br>" $ lines $ show b
+mixedAmountAsHtml :: MixedAmount -> Html
+mixedAmountAsHtml b = preEscapedString $ addclass $ intercalate "<br>" $ lines $ showMixedAmount b
     where addclass = printf "<span class=\"%s\">%s</span>" (c :: String)
           c = case isNegativeMixedAmount b of Just True -> "negative amount"
                                               _         -> "positive amount"
 
 -------------------------------------------------------------------------------
--- post handlers
+-- POST handlers
 
 postJournalR :: Handler RepHtml
 postJournalR = handlePost
@@ -484,8 +519,8 @@ handleAdd = do
       maybeNonNull = maybe Nothing (\t -> if Data.Text.null t then Nothing else Just t)
       acct1E = maybe (Left "to account required") (Right . unpack) $ maybeNonNull acct1M
       acct2E = maybe (Left "from account required") (Right . unpack) $ maybeNonNull acct2M
-      amt1E = maybe (Left "amount required") (either (const $ Left "could not parse amount") Right . parseWithCtx nullctx someamount . unpack) amt1M
-      amt2E = maybe (Right missingamt)       (either (const $ Left "could not parse amount") Right . parseWithCtx nullctx someamount . unpack) amt2M
+      amt1E = maybe (Left "amount required") (either (const $ Left "could not parse amount") Right . parseWithCtx nullctx amount . unpack) amt1M
+      amt2E = maybe (Right missingmixedamt)       (either (const $ Left "could not parse amount") Right . parseWithCtx nullctx amount . unpack) amt2M
       journalE = maybe (Right $ journalFilePath j)
                        (\f -> let f' = unpack f in
                               if f' `elem` journalFilePaths j
@@ -511,20 +546,20 @@ handleAdd = do
                           })
   -- display errors or add transaction
   case tE of
-   Left errs -> do
+   Left errs' -> do
     -- save current form values in session
     -- setMessage $ toHtml $ intercalate "; " errs
-    setMessage [$shamlet|
+    setMessage [shamlet|
                  Errors:<br>
-                 $forall e<-errs
+                 $forall e<-errs'
                   #{e}<br>
                |]
    Right t -> do
     let t' = txnTieKnot t -- XXX move into balanceTransaction
-    liftIO $ do ensureJournalFile journalpath
+    liftIO $ do ensureJournalFileExists journalpath
                 appendToJournalFileOrStdout journalpath $ showTransaction t'
     -- setMessage $ toHtml $ (printf "Added transaction:\n%s" (show t') :: String)
-    setMessage [$shamlet|<span>Added transaction:<small><pre>#{chomp $ show t'}</pre></small>|]
+    setMessage [shamlet|<span>Added transaction:<small><pre>#{chomp $ show t'}</pre></small>|]
 
   redirect (RegisterR, [("add","1")])
 
@@ -566,7 +601,7 @@ handleEdit = do
        setMessage "No change"
        redirect JournalR
      else do
-      jE <- liftIO $ journalFromPathAndString Nothing journalpath tnew
+      jE <- liftIO $ readJournal Nothing Nothing (Just journalpath) tnew
       either
        (\e -> do
           setMessage $ toHtml e
@@ -596,72 +631,72 @@ handleImport = do
   --    redirect JournalR
 
 ----------------------------------------------------------------------
--- | Other view components.
+-- Common page components.
 
 -- | Global toolbar/heading area.
 topbar :: ViewData -> HtmlUrl AppRoute
-topbar VD{..} = [$hamlet|
-<div#topbar
- <a.topleftlink href=#{hledgerorgurl} title="More about hledger"
+topbar VD{..} = [hamlet|
+<div#topbar>
+ <a.topleftlink href=#{hledgerorgurl} title="More about hledger">
   hledger-web
   <br />
   #{version}
  <a.toprightlink href=#{manualurl} target=hledgerhelp title="User manual">manual
  <h1>#{title}
-$maybe m <- msg
- <div#message>#{m}
+$maybe m' <- msg
+ <div#message>#{m'}
 |]
   where
     title = takeFileName $ journalFilePath j
 
--- | Navigation link, preserving parameters and possibly highlighted.
-navlink :: ViewData -> String -> AppRoute -> String -> HtmlUrl AppRoute
-navlink VD{..} s dest title = [$hamlet|
-<a##{s}link.#{style} href=@?{u} title="#{title}">#{s}
-|]
-  where u = (dest, if null q then [] else [("q", pack q)])
-        style | dest == here = "navlinkcurrent"
-              | otherwise    = "navlink" :: Text
+-- -- | Navigation link, preserving parameters and possibly highlighted.
+-- navlink :: ViewData -> String -> AppRoute -> String -> HtmlUrl AppRoute
+-- navlink VD{..} s dest title = [hamlet|
+-- <a##{s}link.#{style} href=@?{u'} title="#{title}">#{s}
+-- |]
+--   where u' = (dest, if null q then [] else [("q", pack q)])
+--         style | dest == here = "navlinkcurrent"
+--               | otherwise    = "navlink" :: Text
 
--- | Links to the various journal editing forms.
-editlinks :: HtmlUrl AppRoute
-editlinks = [$hamlet|
-<a#editformlink href="#" onclick="return editformToggle(event)" title="Toggle journal edit form">edit
-\ | #
-<a#addformlink href="#" onclick="return addformToggle(event)" title="Toggle transaction add form">add
-<a#importformlink href="#" onclick="return importformToggle(event)" style="display:none;">import transactions
-|]
+-- -- | Links to the various journal editing forms.
+-- editlinks :: HtmlUrl AppRoute
+-- editlinks = [hamlet|
+-- <a#editformlink href="#" onclick="return editformToggle(event)" title="Toggle journal edit form">edit
+-- \ | #
+-- <a#addformlink href="#" onclick="return addformToggle(event)" title="Toggle transaction add form">add
+-- <a#importformlink href="#" onclick="return importformToggle(event)" style="display:none;">import transactions
+-- |]
 
 -- | Link to a topic in the manual.
 helplink :: String -> String -> HtmlUrl AppRoute
-helplink topic label = [$hamlet|
+helplink topic label = [hamlet|
 <a href=#{u} target=hledgerhelp>#{label}
 |]
     where u = manualurl ++ if null topic then "" else '#':topic
 
 -- | Search form for entering custom queries to filter journal data.
 searchform :: ViewData -> HtmlUrl AppRoute
-searchform VD{..} = [$hamlet|
-<div#searchformdiv
- <form#searchform.form method=GET
-  <table
-   <tr
-    <td
+searchform VD{..} = [hamlet|
+<div#searchformdiv>
+ <form#searchform.form method=GET>
+  <table>
+   <tr>
+    <td>
      Search:
      \ #
-    <td
-     <input name=q size=70 value=#{q}
-     <input type=submit value="Search"
+    <td>
+     <input name=q size=70 value=#{q}>
+     <input type=submit value="Search">
      $if filtering
       \ #
-      <span.showall
+      <span.showall>
        <a href=@{here}>clear search
      \ #
      <a#search-help-link href="#" title="Toggle search help">help
-   <tr
-    <td
-    <td
-     <div#search-help.help style="display:none;"
+   <tr>
+    <td>
+    <td>
+     <div#search-help.help style="display:none;">
       Leave blank to see journal (all transactions), or click account links to see transactions under that account.
       <br>
       Transactions/postings may additionally be filtered by:
@@ -682,7 +717,7 @@ searchform VD{..} = [$hamlet|
 
 -- | Add transaction form.
 addform :: ViewData -> HtmlUrl AppRoute
-addform vd@VD{..} = [$hamlet|
+addform vd@VD{..} = [hamlet|
 <script type=text/javascript>
  \$(document).ready(function() {
     /* dhtmlxcombo setup */
@@ -700,37 +735,37 @@ addform vd@VD{..} = [$hamlet|
     /* desccombo.setOptionHeight(200); */
  });
 
-<form#addform method=POST style=display:none;
+<form#addform method=POST style=display:none;>
   <h2#contenttitle>#{title}
-  <table.form
-   <tr
-    <td colspan=4
-     <table
-      <tr#descriptionrow
-       <td
+  <table.form>
+   <tr>
+    <td colspan=4>
+     <table>
+      <tr#descriptionrow>
+       <td>
         Date:
-       <td
-        <input.textinput size=15 name=date value=#{date}
-       <td style=padding-left:1em;
+       <td>
+        <input.textinput size=15 name=date value=#{date}>
+       <td style=padding-left:1em;>
         Description:
-       <td
-        <select id=description name=description
-         <option
+       <td>
+        <select id=description name=description>
+         <option>
          $forall d <- descriptions
           <option value=#{d}>#{d}
-      <tr.helprow
-       <td
-       <td
+      <tr.helprow>
+       <td>
+       <td>
         <span.help>#{datehelp} #
-       <td
-       <td
+       <td>
+       <td>
         <span.help>#{deschelp}
    ^{postingfields vd 1}
    ^{postingfields vd 2}
-   <tr#addbuttonrow
-    <td colspan=4
-     <input type=hidden name=action value=add
-     <input type=submit name=submit value="add transaction"
+   <tr#addbuttonrow>
+    <td colspan=4>
+     <input type=hidden name=action value=add>
+     <input type=submit name=submit value="add transaction">
      $if manyfiles
       \ to: ^{journalselect $ files j}
      \ or #
@@ -743,37 +778,38 @@ addform vd@VD{..} = [$hamlet|
   date = "today" :: String
   descriptions = sort $ nub $ map tdescription $ jtxns j
   manyfiles = (length $ files j) > 1
-  postingfields VD{..} n = [$hamlet|
-<tr#postingrow
+  postingfields :: ViewData -> Int -> HtmlUrl AppRoute
+  postingfields _ n = [hamlet|
+<tr#postingrow>
  <td align=right>#{acctlabel}:
- <td
-  <select id=#{acctvar} name=#{acctvar}
-   <option
+ <td>
+  <select id=#{acctvar} name=#{acctvar}>
+   <option>
    $forall a <- acctnames
     <option value=#{a} :shouldselect a:selected>#{a}
  ^{amtfield}
-<tr.helprow
- <td
- <td
+<tr.helprow>
+ <td>
+ <td>
   <span.help>#{accthelp}
- <td
- <td
+ <td>
+ <td>
   <span.help>#{amthelp}
 |]
    where
     shouldselect a = n == 2 && maybe False ((a==).fst) (inAccount qopts)
-    numbered = (++ show n)
-    acctvar = numbered "account"
-    amtvar = numbered "amount"
+    withnumber = (++ show n)
+    acctvar = withnumber "account"
+    amtvar = withnumber "amount"
     acctnames = sort $ journalAccountNamesUsed j
     (acctlabel, accthelp, amtfield, amthelp)
        | n == 1     = ("To account"
                      ,"eg: expenses:food"
-                     ,[$hamlet|
-<td style=padding-left:1em;
+                     ,[hamlet|
+<td style=padding-left:1em;>
  Amount:
-<td
- <input.textinput size=15 name=#{amtvar} value=""
+<td>
+ <input.textinput size=15 name=#{amtvar} value="">
 |]
                      ,"eg: $6"
                      )
@@ -785,28 +821,28 @@ addform vd@VD{..} = [$hamlet|
 
 -- | Edit journal form.
 editform :: ViewData -> HtmlUrl AppRoute
-editform VD{..} = [$hamlet|
-<form#editform method=POST style=display:none;
- <h2#contenttitle>#{title}
- <table.form
+editform VD{..} = [hamlet|
+<form#editform method=POST style=display:none;>
+ <h2#contenttitle>#{title}>
+ <table.form>
   $if manyfiles
-   <tr
-    <td colspan=2
+   <tr>
+    <td colspan=2>
      Editing ^{journalselect $ files j}
-  <tr
-   <td colspan=2
+  <tr>
+   <td colspan=2>
     <!-- XXX textarea ids are unquoted journal file paths here, not valid html -->
     $forall f <- files j
-     <textarea id=#{fst f}_textarea name=text rows=25 cols=80 style=display:none; disabled=disabled
+     <textarea id=#{fst f}_textarea name=text rows=25 cols=80 style=display:none; disabled=disabled>
       #{snd f}
-  <tr#addbuttonrow
-   <td
+  <tr#addbuttonrow>
+   <td>
     <span.help>^{formathelp}
-   <td align=right
-    <span.help
+   <td align=right>
+    <span.help>
      Are you sure ? This will overwrite the journal. #
-    <input type=hidden name=action value=edit
-    <input type=submit name=submit value="save journal"
+    <input type=hidden name=action value=edit>
+    <input type=submit name=submit value="save journal">
     \ or #
     <a href="#" onclick="return editformToggle(event)">cancel
 |]
@@ -817,30 +853,30 @@ editform VD{..} = [$hamlet|
 
 -- | Import journal form.
 importform :: HtmlUrl AppRoute
-importform = [$hamlet|
-<form#importform method=POST style=display:none;
- <table.form
-  <tr
-   <td
-    <input type=file name=file
-    <input type=hidden name=action value=import
-    <input type=submit name=submit value="import from file"
+importform = [hamlet|
+<form#importform method=POST style=display:none;>
+ <table.form>
+  <tr>
+   <td>
+    <input type=file name=file>
+    <input type=hidden name=action value=import>
+    <input type=submit name=submit value="import from file">
     \ or #
-    <a href="#" onclick="return importformToggle(event)" cancel
+    <a href="#" onclick="return importformToggle(event)">cancel
 |]
 
 journalselect :: [(FilePath,String)] -> HtmlUrl AppRoute
-journalselect journalfiles = [$hamlet|
-<select id=journalselect name=journal onchange="editformJournalSelect(event)"
+journalselect journalfiles = [hamlet|
+<select id=journalselect name=journal onchange="editformJournalSelect(event)">
  $forall f <- journalfiles
   <option value=#{fst f}>#{fst f}
 |]
 
 nulltemplate :: HtmlUrl AppRoute
-nulltemplate = [$hamlet||]
+nulltemplate = [hamlet||]
 
 ----------------------------------------------------------------------
--- utilities
+-- Utilities
 
 -- | A bundle of data useful for hledger-web request handlers and templates.
 data ViewData = VD {
@@ -850,9 +886,9 @@ data ViewData = VD {
     ,today        :: Day        -- ^ today's date (for queries containing relative dates)
     ,j            :: Journal    -- ^ the up-to-date parsed unfiltered journal
     ,q            :: String     -- ^ the current q parameter, the main query expression
-    ,m            :: Matcher    -- ^ a matcher parsed from the q parameter
+    ,m            :: Query    -- ^ a query parsed from the q parameter
     ,qopts        :: [QueryOpt] -- ^ query options parsed from the q parameter
-    ,am           :: Matcher    -- ^ a matcher parsed from the accounts sidebar query expr ("a" parameter)
+    ,am           :: Query    -- ^ a query parsed from the accounts sidebar query expr ("a" parameter)
     ,aopts        :: [QueryOpt] -- ^ query options parsed from the accounts sidebar query expr
     ,showpostings :: Bool       -- ^ current p parameter, 1 or 0 shows/hides all postings where applicable
     }
@@ -925,6 +961,7 @@ getMessageOr mnewmsg = do
   oldmsg <- getMessage
   return $ maybe oldmsg (Just . toHtml) mnewmsg
 
+numbered :: [a] -> [(Int,a)]
 numbered = zip [1..]
 
 dayToJsTimestamp :: Day -> Integer
