@@ -1,6 +1,5 @@
 module Hledger.Web.Import
     ( module Prelude
-    , module Hledger.Web.Foundation
     , (<>)
     , Text
     , module Data.Monoid
@@ -12,8 +11,6 @@ import Data.Monoid (Monoid (mappend, mempty, mconcat))
 import Control.Applicative ((<$>), (<*>), pure)
 import Data.Text (Text)
 
-import Hledger.Web.Foundation
-
 infixr 5 <>
 (<>) :: Monoid m => m -> m -> m
 (<>) = mappend
